#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

// Definiciones

#define WHITE 0
#define GRAY 1
#define BLACK 2
#define MAX_NODES 1000
#define oo 1000000000


int n;  // numero de nodos
int e;  // numero de arcos
float capacity[MAX_NODES][MAX_NODES]; // matriz de capacidades
float flow[MAX_NODES][MAX_NODES];     // matriz de flujos
int color[MAX_NODES]; // colores para BFS               
int pred[MAX_NODES];  // caminos de aumentos

float min (float x, float y) {
    return x<y ? x : y;
}
int max (int x, int y) {
    return x>y ? x : y;
}

// Cola requerida por la funcion BFS.

int head,tail;
int q[MAX_NODES+2];

void enqueue (int x) {
    q[tail] = x;
    tail++;
    color[x] = GRAY;
}

int dequeue () {
    int x = q[head];
    head++;
    color[x] = BLACK;
    return x;
}

//BfS para el camino de aumento

int bfs (int start, int target) {
    int u,v;
    for (u=0; u<n; u++) {
		color[u] = WHITE;
    }   
    head = tail = 0;
    enqueue(start);
    pred[start] = -1;
    while (head!=tail) {
		u = dequeue();
        
		/*busca todos los nodos adyacentes (color blanco). Si la capacidad de u a v
		en la red residual es positiva, se encola v*/
		for (v=0; v<n; v++) {
	    	if ((color[v]==WHITE) && ((capacity[u][v]-flow[u][v])>0)) {
				enqueue(v);
				pred[v] = u;
	   		}
		}
    }
    /*si el color del nodo actual es negro, significa que fue alcanzado*/
	//printf("bfs condicion %d\n", color[target]==BLACK);
    return color[target]==BLACK;
}

// Algoritmo Ford-Fulkerson

float max_flow (int source, int sink) {
    int i,j,u;
    // inicializacion de la matriz de flujo
    float max_flow = 0;
    for (i=0; i<n; i++) {
		for (j=0; j<n; j++) {
	    	flow[i][j] = 0;
		}
    }
    // Mientrs existe un camino de aumento, incrementa el flujo a traves de esta ruta
    while (bfs(source,sink)) {
        // Determinacion del incremento en el flujo
		float increment = oo;
		for (u=n-1; pred[u]>=0; u=pred[u]) {
			increment = min(increment, capacity[pred[u]][u]-flow[pred[u]][u]);
		}
			// incremento del flujo.
		for (u=n-1; pred[u]>=0; u=pred[u]) {
			flow[pred[u]][u] += increment;
			flow[u][pred[u]] -= increment;
		}
		max_flow += increment;
    }
    
    return max_flow;
}

//lectura del archivo de entrada y el main()

int read_input_file(char *filename) {
    int a,b,i,j;
    float c;
	FILE* input = fopen(filename, "r");
	
	if (input == NULL) {
		printf("Error al abrir el archivo del grafo\n");
		return -1;
	}
	// inicializacion de la matriz de capacidad
    for (i=0; i<MAX_NODES; i++) {
		for (j=0; j<MAX_NODES; j++) {
	    	capacity[i][j] = 0;
			flow[i][j] = 0;
		}
    }
	e = n = 0;
	while (fscanf(input,"%d %d %f", &a, &b, &c) == 3) {
		capacity[a][b] = c;
		n = max(n,max(a,b));
		e++;
		//printf("arco %d %d %f\n", a,b,c);
    }
	n++;
	//printf("nodos=%d arcos=%d\n", n, e);
    fclose(input);
	
	return 0;
}

int main (int argc, char **argv) {
	int a, b;
	
	if (argc < 2) {
		printf("Especifique el archivo con el grafo\n");
		return -1;
	}
	
    if (read_input_file(argv[1]))
		return -1;
	
    fprintf(stderr, "Flujo maximo total = %f\n", max_flow(0, n-1));
	
	/* Generar salida */
	for (a=0; a<n; a++) {
		for (b=0; b<n; b++) {
			if (capacity[a][b] > 0)
				printf("%d %d %f/%f\n", a, b, flow[a][b], capacity[a][b]);				
		}
	}
	
    return 0;
}
